#include "WebCache.h"

/**
 * File: WebCache.cpp
 * Task: Implement WebCache.h
 * Desc: Executing the web caching to the memory.
 *       It caches HTML file and objects that belong to HTML file
 *       (things such as JPEG, GIF, etc). It uses Least Recently 
 *       Used (LRU) algorithm to replace the cache page. 
 *       It depends heavily on struct CACHE (look at WebCache.h)
 *       To fully used CACHE to the fullest extend, one needs to
 *       replace char objects[20] --> to some sort of pointer to
 *       binary file (JPEG, GIF) either by the mean of ifstream,
 *       or other possible means.
 * Used: 
 *       // normal usage
 *       WebCache wb;
 *       CACHE my_cache; // then fill up my_cache
 *       wb.store(my_cache);
 *
 *       // retrieval of cache
 *       CACHE my_cache;
 *       char hostname = some_host_name;
 *       int index = wb.searchCache(hostname);
 *       if(index < 0)
 *          meaning cache based on hostname not exist in WebCache
 *       else
 *          my_cache = retrieveCache(index) 
 *
 * Auth: Edison Chindrawaly
 * Vers: 1.0
 * Date: Nov 12, 2001
 */

/**
 * default constructor: initialize the space in memory
 * @param  none
 * @return none
 */
WebCache::WebCache()
{
 clean();
}

/**
 * default destructor: clean the memory space
 * @param  none
 * @return none
 */
WebCache::~WebCache() 
{
 clean();
}

/**
 * clean(): help constructor & destructor to claim the space
 *          in the memory for the program.
 * @param  none
 * @return none
 */
void WebCache::clean()
{
 fileCache = 0;
 for(int i=0; i<MAX; i++)
 {
  myCache[i].age = 0;
  myCache[i].host[0] = '\0';
  myCache[i].httpFile[0] ='\0';
  myCache[i].objects[0] = '\0';
 }
}

/**
 * examine(): implement LRU algorithm by looking for the
 *            the smallest value of age. Then return the index
 *            number of the smallest value of age.
 * @param  none
 * @return int index for the smallest value of age
 */
int WebCache::examine()
{
 int go=0, index=0;
 do
 {
  for(;go < MAX; go++)
  {
   if(myCache[go].age > myCache[go+1].age)
    go = go + 1;
   index = go;
  }
 }while(go < MAX);
 return index;
}

/**
 * retrieveCache(): to retrieveCache based on index position
 * @param  int index 
 * @return CACHE
 */
CACHE WebCache::retrieveCache(int index)
{

 myCache[index].age++;
 CACHE mCache = myCache[index];
 return mCache; 
}

/**
 * replaceCache(): 
 *   To replace cache 
 *   if the duplicate cache is found (option != 0), 
 *      replace it and increase age. 
 *   else put the new cache in the empty slot determine 
 *        by counter: fileCache.
 *   
 * @param  int index  - position of cache
 *         int option - 0: no duplicate cache found
 *                      1: duplicate cache found
 *         CACHE mCache Cache to be enter into memory
 * @return int - 0: successful
 *               1: error of exceeding the max hostname
 *               2: error of exceeding the max length of html 
 */
int WebCache::replaceCache(int index, int option, CACHE mCache)
{

 if(option == 0)
   myCache[index].age = 0;
 else
   myCache[index].age++;

 if(strlen(mCache.host) > LENGTH)
   return 1; // error
 strcpy(myCache[index].host,mCache.host);

 if(strlen(mCache.httpFile) > LENGTH)
   return 2; // error
 strcpy(myCache[index].httpFile,mCache.httpFile);

 //-----------------------------
 //then copy objects to objects.
 //-----------------------------

 return 0; // successful replacement
}

/**
 * storeCache():
 *   To store cache into the memory by checking the array 
 *   of CACHE for the duplication. Look replaceCache() for
 *   detail explaination on how WebCache put cache 
 * @param  CACHE mCache - Cache to be store
 * @return int error = 0: no error; otherwise error occurs
 */
int WebCache::storeCache(CACHE mCache)
{

 int error = -1;
 int found = searchCache(mCache.host);

 if(found > error)
 {
   error = replaceCache(found,found,mCache);
   reportError(error);
 }
 else
 {
  if(fileCache != MAX)
  {
    error = replaceCache(fileCache,0,mCache);
    if(error == 0)
      fileCache++;
    else
      reportError(error);
  }
  else
  {
    error = replaceCache(examine(),0,mCache);
    reportError(error);
  } 
 }

 return error;
}

/**
 * searchCache(): 
 *   To search for duplicate cache in array of CACHE. It uses
 *   hostname in CACHE.host and case compare it with the given 
 *   value passed by calling method.
 * @param  char* host - contains the host name of the CACHE 
 * @return int -1 if no duplicate found; else return the
 *         index value of the CACHE position in the array 
 */
int WebCache::searchCache(char* host)
{

 for(int i=0;i<fileCache;i++)
 {
  if(strcasecmp(host,myCache[i].host) == 0)
    return i;  
 } 
 return -1;
}

/**
 * reportError(): Error reporting mechanism
 * @param  int error
 * @return none
 */
void WebCache::reportError(int error)
{
 switch(error)
 {
  case 0: break;
  case 1:
          cout<<"WebCache ERROR: ";
          cout<<"host name is longer than LENGTH: "<<LENGTH<<endl;
          break;
  case 2:
          cout<<"WebCache ERROR: ";
          cout<<"httpFile exceeds the MAXSIZE: "<<MAXSIZE<<endl;
  default:break;
 }

}
